function [sacc, nsac, xx, mov_type] = detect_saccades(x, Tdur, Fs, Tvel, Tvel2, Tisi, Tpvel)

% [sacc, nsac, xx, mov_type] = detect_saccades(x, Tdur, Fs, Tvel, Tvel2, Tisi, Tpvel)
%
% Detect saccades using velocity thresholding
%
% Input:
%   x    : input data, [x_position y_position time], in degree and second
%   Tdur : saccade minimum duration (default is 12 msec)
%   Fs   : sampling rate of the data (default is 500 Hz)
%   Tvel : velocity threshold for saccade detection (default is 30 deg/s)
%   Tvel2: velocity threshold for saccade endpoint  (default is 30 deg/s)
%   Tisi : threshold for minimal inter-saccadic interval (default is none)
%   Tpvel: threshold for non-physiological saccade (default is none)
% 
% Output:
%   sacc     : detected saccades, [startpoint endpoint sacc_properties]
%   nsac     : number of detected saccades
%   xx       : [time x_pos. y_pos. x_vel. y_vel. x_acc. y_acc. vel acc]
%   move_type: label of eye movement, 1 for saccade, 0 for fixation

% Reference
% Detection of normal and slow saccades using implicit piecewise polynomial approximation
% Weiwei Dai, Ivan Selesnick, John-Ross Rizzo, Janet Rucker, Todd Hudson
%
% Tandon School of Engineering and School of Medicine
% New York University
% New York, USA
%
% Contact: 
% Ivan Selesnick selesi@nyu.edu
% Weiwei Dai wd471@nyu.edu


% default setting of parameters
if ~exist('Tdur', 'var')
    Tdur = 12;
end
if ~exist('Fs', 'var')
    Fs = 500;
end
if ~exist('Tvel', 'var')
    Tvel = 30;
end
if ~exist('Tvel2', 'var')
    Tvel2 = [];
end
if ~exist('Tisi', 'var')
    Tisi = [];
end
if ~exist('Dpvel', 'var')
    Tpvel = [];
end

if length(Tvel) == 1                % same velocity threshold for x and y
    Tvel = ones(1,2)*Tvel;
end

N = size(x,1);
xx = nan(N,9);
t = 1000*(x(:,end)-x(1,end));
xx(:,1) = t;                        % time in millisecond

pos_x = x(:,1);
xx(:,2) = pos_x;                    % x position in degree
pos_y = x(:,2);
xx(:,3) = pos_y;                    % y position in degree

vel_x = calculateV(pos_x, Fs);
xx(:,4) = vel_x;
vel_y = calculateV(pos_y, Fs);
xx(:,5) = vel_y;
acc_x = calculateA(pos_x, Fs);
xx(:,6) = acc_x;
acc_y = calculateA(pos_y, Fs);
xx(:,7) = acc_y;

vel = sqrt(vel_x.^2 + vel_y.^2);
acc = calculateV(vel, Fs);
xx(:,8) = vel;
xx(:,9) = acc;

% detect saccades
if isempty(Tvel)
else
    outside_idx = (vel_x/Tvel(1)).^2 + (vel_y/Tvel(2)).^2;
    outside_idx = (outside_idx>=1);
    start_idx = find(diff([0; outside_idx])==1);
    end_idx = find(diff([outside_idx; 0])==-1);
end
    
% post-processing and collect saccade information
nsac1 = length(start_idx); 
sacc = cell(nsac1,3);
saccfilter = ones(nsac1,1);
for i = 1:nsac1
    saccPara = [];
    % if this saccade is included in the previous one
    if i ~= 1
        if start_idx(i) <= max(end_idx(1:i-1))
            saccfilter(i) = 0;
            continue
        end
    end
    % if this saccade follow a blink (noise) within 10 samples
    if i ~= 1
        if sum(isnan(vel(max(1,start_idx(i)-10):start_idx(i)))) > 0
            saccfilter(i) = 0;
            continue
        end
    end
    % second velocity threshold for the end of a saccade
    if ~isempty(Tvel2)
        temp_idx = end_idx(i):min(end_idx(i)+250, N);
        if isempty(find((vel(temp_idx) < Tvel2) | isnan(vel(temp_idx)),1))
            end_idx(i) = end_idx(i) + length(temp_idx) - 1;
        else
            end_idx(i) = end_idx(i) + find((vel(temp_idx) < Tvel2) | isnan(vel(temp_idx)),1) - 1;
        end
    end
    % if this saccade followed by a blink (noise) within 10 samples
    if i ~= nsac1
        if sum(isnan(vel(end_idx(i):min(N,end_idx(i)+10)))) > 0
            saccfilter(i) = 0;
            continue
        end
    end
    % if this saccade satisfies minimal duration
    dur = t(end_idx(i)) - t(start_idx(i));
    if dur < Tdur
        saccfilter(i) = 0;
        continue
    end
    sacc{i,1} = start_idx(i);
    sacc{i,2} = end_idx(i);
    saccPara.dur = dur;                                         % duration in millisecond
    % if this saccade satisfies minimal inter-saccadic interval
    if i ~= 1
        k = i - 1;
        while saccfilter(k) == 0 && k > 1
            k = k - 1;
        end
        if saccfilter(k) == 0 && k == 1
            isi = NaN;
        else
            isi = t(start_idx(i)) - t(end_idx(k));              % ISI in millisecond
            if ~isempty(Tisi)
                if isi < Tisi
                    saccfilter(i) = 0;
                    continue
                end
            end
        end
    else
        isi = NaN;
    end
    saccPara.isi = isi;
    saccPara.amp_x = pos_x(end_idx(i))-pos_x(start_idx(i));     % positive for rightwards
    saccPara.amp_y = pos_y(end_idx(i))-pos_y(start_idx(i));     % positive for upwards
    saccPara.amp = sqrt(saccPara.amp_x^2 + saccPara.amp_y^2);   % always positive
    saccPara.pvel = max(vel(start_idx(i):end_idx(i)));          % peak velocity
    
    % if this saccade is non-physiological
    if ~isempty(Tpvel)
        if abs(saccPara.pvel) > Tpvel
            saccfilter(i) = 0;
            continue
        end
    end
    sacc{i,3} = saccPara;
end

saccfilter = logical(saccfilter);
sacc = sacc(saccfilter, :);
nsac = size(sacc,1);

mov_type = zeros(N,1);
for i = 1:nsac
    mov_type(sacc{i,1}:sacc{i,2}) = 1;
end

end