function [sacc, nsac, xx, mov_type] = eyetracking_denoising(DATA, Tvel, Fs, lam, Tdur)

% [sacc, nsac, xx, mov_type] = eyetracking_denoising(DATA, Tvel, Fs, lam, Tdur)
%
% Denoise eye tracking data and detect saccades using velocity thresholding
%
% Input:
%    DATA: input data, [x_position y_position time], in degree and second
%    Tvel: velocity threshold for saccade detection (default is 30 deg/s)
%    Fs  : sampling rate (default is 500 Hz)
%    lam : smoothing tuner (default is 1)
%    Tdur: minimal duration of a saccade (default is 12 msec)

% Reference
% Detection of normal and slow saccades using implicit piecewise polynomial approximation
% Weiwei Dai, Ivan Selesnick, John-Ross Rizzo, Janet Rucker, Todd Hudson
%
% Tandon School of Engineering and School of Medicine
% New York University
% New York, USA
%
% Contact: 
% Ivan Selesnick selesi@nyu.edu
% Weiwei Dai wd471@nyu.edu

if ~exist('Fs', 'var')
    Fs = 500;
end
if ~exist('lam', 'var')
    lam = 1;
end
if ~exist('Tvel', 'var')
    Tvel = 30;
end
if ~exist('Tdur', 'var')
    Tdur = 12;
end

T = 10;         % velcolity threshold used in noise estimation
rDATA = DATA;   % raw noisy data
nDATA = DATA;   % processed data

Nit = 30;

for i = 1:2                             % process x and y separately
    Y = rDATA(:,i);
    if any(Y)
        N = length(Y);
        DY = conv(Y, Fs*[1 0 -1]/2, 'same');
        DY = movmean(DY, 60*Fs/1000+1);	% fc is about 10 Hz
        temp1 = (abs(DY)>=T);           % 1 for saccades, 0 for others
        [aa, bb] = mybwlabel(temp1);  
        for aaa = 1:bb
            cc = find(aa==aaa);
            if length(cc) < 12
                temp1(cc) = 0;
            end
        end
        idx = temp1(2:end) - temp1(1:end-1);
        SP = find(idx==1) + 1;
        if temp1(1) == 1
            SP = [1; SP];
        end
        EP = find(idx==-1) + 1;
        if temp1(end) == N-1
            EP = [EP; N];
        end
        for EPi = 1:length(EP)-1
            if (SP(EPi+1) - EP(EPi))<20
                temp1(EP(EPi):SP(EPi+1)) = 1;
            end
        end
        
        y = Y;
        dy = DY;
        temp2 = temp1;                  % 1 for saccades, 0 for others
        
        % estimate noise
        [fixation, nfix] = mybwlabel(temp2==0);
        y_fix = y;
        for ii = 1:nfix
            fix_idx = find(fixation == ii);
            y_fix(fix_idx) = y_fix(fix_idx) - mean(y_fix(fix_idx), 'omitnan');
        end
        sigma = std(y_fix(~temp2), 'omitnan');
        
        temp3 = abs(dy);                % velocities for saccades
        temp3(~temp2) = 0;
        [temp4, nsacc] = mybwlabel(temp3~=0);        
        if nsacc == 0
            amp_avg = 1;
            dur_avg = 0;
        else
            amp_cum = sum(temp3)/Fs;
            amp_avg = amp_cum/nsacc;
            dur = zeros(nsacc,1);
            for nsaci = 1:nsacc
                dur(nsaci) = sum((temp4 == nsaci));
            end
            dur_avg = mean(dur)/Fs;
        end
        
        if Fs <= 500
            alpha = 8*sigma*(0.002*Fs);
            beta  = 4*sigma*sqrt(amp_avg)*exp(5*dur_avg)*(0.002*Fs);
        else
            alpha = 8*sigma*(Fs/2500+0.8);
            beta  = 4*sigma*sqrt(amp_avg)*exp(5*dur_avg)*(Fs/2500+0.8);
        end
        alpha = lam*alpha;
        beta  = lam*beta;
        
        x = cgtv(y, alpha, beta, Nit);
        
        nDATA(:,i) = x;        
    end
end

% detect saccades using velocity thresholding
Tvel2 = 10; Tisi = 40; Tpvel = 800;
[sacc, nsac, xx, mov_type] = detect_saccades(nDATA, Tdur, Fs, Tvel, Tvel2, Tisi, Tpvel);

end



function [y, n] = mybwlabel(x)

% [y, n] = MYBWLABEL(x)
%
% Label connected ones in a array
%
% Input:
%   x  : input array
% 
% Output:
%   y  : labeled input array in sequence, label 1 for first connected ones,
%        label 2 for second connected ones, ...
%   n  : number of connected ones found in array

y = cumsum([1;abs(diff(x))]);
if x(1)
    y(mod(y,2)==0) = 0;
    y(mod(y,2)==1) = (y(mod(y,2)==1)+1)/2;
else
    y(mod(y,2)==1) = 0;
    y(mod(y,2)==0) = y(mod(y,2)==0)/2;
end
n = max(y); 

end



